from typing import Callable, Dict

import azure_data_factory_testing_framework.functions.functions_collection_implementation as collection_functions
import azure_data_factory_testing_framework.functions.functions_conversion_implementation as conversion_functions
import azure_data_factory_testing_framework.functions.functions_date_implementation as date_functions
import azure_data_factory_testing_framework.functions.functions_logical_implementation as logical_functions
import azure_data_factory_testing_framework.functions.functions_math_implementation as math_functions
import azure_data_factory_testing_framework.functions.functions_string_implementation as string_functions


class FunctionsRepository:
    functions: Dict[str, Callable] = {
        "add": None,
        "addDays": None,
        "addHours": None,
        "addMinutes": None,
        "addSeconds": None,
        "addToTime": None,
        "and": logical_functions.and_,
        "array": conversion_functions.array,
        "base64": conversion_functions.base64,
        "base64ToBinary": conversion_functions.base64_to_binary,
        "base64ToString": conversion_functions.base64_to_string,
        "binary": conversion_functions.binary,
        "bool": conversion_functions.bool_,
        "coalesce": conversion_functions.coalesce,
        "concat": string_functions.concat,
        "contains": collection_functions.contains,
        "convertFromUtc": None,
        "convertTimeZone": None,
        "convertToUtc": None,
        "createArray": conversion_functions.create_array,
        "dataUri": conversion_functions.data_uri,
        "dataUriToBinary": conversion_functions.data_uri_to_binary,
        "dataUriToString": conversion_functions.data_uri_to_string,
        "dayOfMonth": None,
        "dayOfWeek": None,
        "dayOfYear": None,
        "decodeBase64": conversion_functions.decode_base64,
        "decodeDataUri": conversion_functions.decode_data_uri,
        "decodeUriComponent": conversion_functions.decode_uri_component,
        "div": math_functions.div,
        "encodeUriComponent": conversion_functions.encode_uri_component,
        "empty": collection_functions.empty,
        "endsWith": string_functions.ends_with,
        "equals": logical_functions.equals,
        "first": collection_functions.first,
        "float": conversion_functions.float_,
        "formatDataTime": None,
        "getFutureTime": None,
        "getPastTime": None,
        "greater": logical_functions.greater,
        "greaterOrEquals": logical_functions.greater_or_equals,
        "guid": string_functions.guid,
        "if": logical_functions.if_,
        "indexOf": string_functions.index_of,
        "int": conversion_functions.int_,
        "json": conversion_functions.json,
        "intersection": collection_functions.intersection,
        "join": collection_functions.join,
        "last": collection_functions.last,
        "lastIndexOf": string_functions.last_index_of,
        "length": collection_functions.length,
        "less": logical_functions.less,
        "lessOrEquals": logical_functions.less_or_equals,
        "max": None,
        "min": None,
        "mod": None,
        "mul": None,
        "not": logical_functions.not_,
        "or": logical_functions.or_,
        "rand": None,
        "range": None,
        "replace": string_functions.replace,
        "skip": collection_functions.skip,
        "split": string_functions.split,
        "startOfDay": None,
        "startOfHour": None,
        "startOfMonth": None,
        "startsWith": string_functions.starts_with,
        "string": conversion_functions.string,
        "sub": math_functions.sub,
        "substring": string_functions.substring,
        "substractFromTime": None,
        "take": collection_functions.take,
        "ticks": date_functions.ticks,
        "toLower": string_functions.to_lower,
        "toUpper": string_functions.to_upper,
        "trim": string_functions.trim,
        "union": collection_functions.union,
        "uriComponent": conversion_functions.uri_component,
        "uriComponentToBinary": conversion_functions.uri_component_to_binary,
        "uriComponentToString": conversion_functions.uri_component_to_string,
        "utcNow": date_functions.utcnow,
        "xml": conversion_functions.xml,
        "xpath": conversion_functions.xpath,
    }

    @staticmethod
    def register(function_name: str, function: Callable) -> None:
        FunctionsRepository.functions[function_name] = function
